#include <vector>
#include <set>
#include <list>
#include <queue>
#include <stack>
#include <iterator>
#include <iostream>
#include <iomanip>
#include <fstream>
#include <cmath>

//#define	TEST
#define iter(container, it) for(typeof(container.begin()) it = container.begin(); it != container.end(); it++)

using namespace std;

class Edge{
	private:
		int p,q,flow,cap;
	public:
		Edge(int pp, int qq, int capp) : p(pp), q(qq), cap(capp), flow(0) {}
		int P() const {return p;}
		int Q() const {return q;}
		bool from(int v) {return (p==v);}
		int other(int v) {return (v==P()) ? Q() : P();}
		int getFlow() {return flow;}
		int getCap() {return cap;}
		void incFlow(int val) {flow+=val;} 
		void incCap(int val) {cap+=val;} 
		int rCapFrom(int v){return from(v) ? cap-flow : flow;}
		int rCapTo(int v){return from(v) ? flow : cap-flow;}
		void addRFlowFrom(int v, int val){flow += from(v) ? val : -val;}
		void addRFlowTo(int v, int val){flow += from(v) ? -val : val;}
};

class Graph{
	private:
		bool someFlow;
		int vertex_count;

		//flow
		vector<Edge*> st; //spanning tree
		vector<int> height; // height funkcia pre preflow push algoritmus

		//kontajney pre prioritnu frontu obsahujuce vrchol, max.pretlacitelny flow & prichodziu hranu
		struct PriV{
			int vertex;
			int priority;
			Edge* e;
			PriV(int v, int val, Edge* ee) : vertex(v), priority(val), e(ee) {}
			bool operator<(const PriV& x) const {return priority < x.priority;}
			bool operator>(const PriV& x) const {return priority > x.priority;}
		};
		//dvojice oprotivsich hran
		struct op{
			Edge* x, *y;
			op(Edge* a, Edge* b) : x(a), y(b) {}
		};
		vector<op> opposite; //vector oprotivsich dvojic

		//pomocne funkcie
		void computeHeights(int);
		void testOpposite();
		bool bfs(int, int);
		bool dfs(int, int);
		bool pfs(int, int);
		void augment(int, int);


	public:
		vector< list<Edge*> > adj;
#ifdef TEST
		//premenne pre testovacie vypocty
		//FF
		long long augC;
		double augLenght;
		//PreflowPush
		long long pushC;
		long long relabelC;
		//spolocne
		long long edgesC;
#endif
		
		Graph(int v) : vertex_count(v), adj(v), st(v), height(v), someFlow(0) {}
		int VCnt() const {return vertex_count;}
		void insert(Edge *);
		void insertO(Edge *);
		void remove(Edge *);
		void show();

		//vystupy do formatu .dot citatelneho prgramom graphviz

		void graphviz(char in[], int s, int t);
		void graphviz_di(char in[], int s, int t);

		//flow

		void clearFlow();

		int flow(int);
		int cost();
		bool flowCheck(int, int);

		int maxFlowBFS(int, int);
		int maxFlowDFS(int, int);
		int maxFlowPFS(int, int);
		int preflowPush(int, int);
		int highestPreflowPush(int s, int t);
		int lowestPreflowPush(int s, int t);

};


int Graph::flow(int v){
	int sum=0;
	for(list<Edge*>::iterator i=adj[v].begin();i!=adj[v].end();i++){
		sum+=(*i)->from(v) ? (*i)->getFlow() : -(*i)->getFlow();
	}
	return sum;
}

bool Graph::flowCheck(int s, int t){
	for (int i=0;i<VCnt();i++)
		if (i!=s && i!=t && flow(i)!=0) return false;
	if (flow(s)<0) return false;
	if (flow(s) + flow(t) != 0) return false;
	return true;
}

void Graph::insert(Edge* e){
	bool present = false;
	//ak najde hranu prida kapacitu, inak prida hranu na obe miesta cost ostane po tej povodnej
	for(list<Edge*>::iterator i=adj[e->P()].begin();i!=adj[e->P()].end();i++){
		Edge* j = *i;
		if (j->P()==e->Q() && j->Q()==e->P()){
			opposite.push_back(op(j,e));
		}		
		if (j->from(e->P()) && j->Q()==e->Q()) { 
			j->incCap(e->getCap());
			present = true;
		}
	}
	if (!present) {
		adj[e->P()].push_front(e);
		adj[e->Q()].push_front(e);
	}
}

//insert v pripade ze je jasne ze nepojde o nasobnu hranu
void Graph::insertO(Edge* e){ 
	adj[e->P()].push_front(e);
	adj[e->Q()].push_front(e);
}


void Graph::remove(Edge* e){
	int x=-1;
	for (int i=0;i<opposite.size();i++){
		if (opposite[i].x==e || opposite[i].y==e) {
			x=i;	
		}
	}
	if (x>=0){
		opposite[x]=opposite[opposite.size()-1];
		opposite.pop_back();
	}
	int e1=e->P();
	int e2=e->Q();
	for(list<Edge*>::iterator i=adj[e1].begin();i!=adj[e1].end();i++){
		if ((*i)==e) adj[e1].erase(i);
	}		
	for(list<Edge*>::iterator i=adj[e2].begin();i!=adj[e2].end();i++){
		if ((*i)==e) adj[e2].erase(i);
	}		
	(*e).~Edge();
}


void Graph::graphviz_di(char in[]="graph.dot",int s=-1,int t=-1){
	int out=0;
	ofstream fout (in);
	fout<<"digraph {"<<endl;
	fout<<"  node [shape = point]"<<endl;
	if (s!=-1)
		fout<<s<<" [fontsize = 30 label = \"s\" shape = circle width = 0.5 height = 0.5]"<<endl;
	if (t!=-1)
		fout<<t<<" [fontsize = 30 label = \"t\" shape = circle width = 0.5 height = 0.5]"<<endl;
	for (int j=0;j<VCnt();j++){
		for(list<Edge*>::iterator i=adj[j].begin();i!=adj[j].end();i++){
			if ((*i)->from(j)) {
				fout<< (*i)->P() << " -> " << (*i)->Q();
				if ((*i)->getFlow()) {
					fout<<"  [style = \"setlinewidth(" << (((*i)->getFlow())/15)+2 << ")\"]";
					out++;
				}
				else fout<<"  [style = dotted]";
//				if ((*i)->getFlow()) fout<<"  [style = "setlinewidth(<<(*i)->getFlow())<<)"]";
//				else fout<<"  [style = dotted]";
				fout<<endl;
			}
		}
	}
	fout<<"}"<<endl;
	//return out;
}

void Graph::graphviz(char in[]="graph.dot",int s=-1,int t=-1){
	ofstream fout (in);
	fout<<"graph {"<<endl;
	for (int j=0;j<VCnt();j++){
		for(list<Edge*>::iterator i=adj[j].begin();i!=adj[j].end();i++){
			if ((*i)->from(j) && ((*i)->P() > (*i)->Q()) ) {
				fout<< (*i)->P() << " -- " << (*i)->Q();
				if ((*i)->getFlow()) fout<<"[style = bold]";
				else fout<<"[style = dotted]";

				fout<<endl;
			}
			if((*i)->from(j) && ((*i)->P() < (*i)->Q())) {
				if ((*i)->getFlow()) {
					fout<< (*i)->P() << " -- " << (*i)->Q();
					fout<<"[style = bold]";
					fout<<endl;
				}
			}
		}
	}
	fout<<"}"<<endl;
}


void Graph::show(){
	for (int j=0;j<VCnt();j++){
		for(list<Edge*>::iterator i=adj[j].begin();i!=adj[j].end();i++){
			if ((*i)->from(j)) cout << (*i)->P() << " " << (*i)->Q() << " " << (*i)->getCap() << endl;
		}
	}
}

void Graph::testOpposite(){
	for (int i=0;i<opposite.size();i++){
		if (opposite[i].x->getFlow() > opposite[i].y->getFlow()){
			opposite[i].x->incFlow(-opposite[i].y->getFlow());
			opposite[i].y->incFlow(-opposite[i].y->getFlow());
		}
		else {
			opposite[i].y->incFlow(-opposite[i].x->getFlow());
			opposite[i].x->incFlow(-opposite[i].x->getFlow());
		}
	}
}

void Graph::clearFlow(){
	for (int j=0;j<VCnt();j++)
		for(list<Edge*>::iterator i=adj[j].begin();i!=adj[j].end();i++){
			(*i)->incFlow(-(*i)->getFlow());
		}
	someFlow=false;
}


void Graph::augment(int s, int t){
	Edge *e = st[t];
	//najdi max
	int maxF = e->rCapTo(t);
	int act = e->other(t);
	while(act != s){
		e=st[act];
		maxF=min(maxF,e->rCapTo(act));
		act=e->other(act);
	}
	//zniz o max po celej ceste
	e = st[t];
	e->addRFlowTo(t, maxF);
	act = e->other(t);
#ifdef TEST
	augLenght++;
#endif
	while (act !=s){
#ifdef TEST
		augLenght++;
#endif
		e= st[act];
		e->addRFlowTo(act, maxF);
		act = e->other(act);
	}
}

bool Graph::bfs(int s, int t){
	queue<int> q;
	vector<bool> visited(VCnt(),false);
	q.push(s);
	visited[s]=true;
	while (!q.empty()){
		int x=q.front();
		for(list<Edge*>::iterator i=adj[x].begin();i!=adj[x].end();i++){
#ifdef TEST
			edgesC++;
#endif
			if ((*i)->rCapFrom(x) != 0 && !visited[(*i)->other(x)]){
				st[(*i)->other(x)]=*i;
				visited[(*i)->other(x)]=true;
				q.push((*i)->other(x));
				if ((*i)->other(x)==t) return true;
			}
		}
		q.pop();
	}
	return false;
}

bool Graph::dfs(int s, int t){
	stack<int> q;
	vector<bool> visited(VCnt(),false);
	q.push(s);
	visited[s]=true;
	while (!q.empty()){
		int x=q.top();
		q.pop();
		for(list<Edge*>::iterator i=adj[x].begin();i!=adj[x].end();i++){
#ifdef TEST
			edgesC++;
#endif
			if ((*i)->rCapFrom(x) != 0 && !visited[(*i)->other(x)]){
				st[(*i)->other(x)]=*i;
				visited[(*i)->other(x)]=true;
				q.push((*i)->other(x));
				if ((*i)->other(x)==t) return true;
			}
		}
	}
	return false;
}

bool Graph::pfs(int s, int t){
	if (s==t) return false;
	vector<bool> visited(VCnt(),false);
	priority_queue<PriV> q;
	q.push(PriV(s,INT_MAX, new Edge(s,s,0)));
	while (!q.empty()){
		PriV x=q.top();
		visited[x.vertex]=true;
		st[x.vertex]=x.e;
		if (x.vertex==t) return true;
		for(list<Edge*>::iterator i=adj[x.vertex].begin();i!=adj[x.vertex].end();i++){
#ifdef TEST
			edgesC++;
#endif
			if ((*i)->rCapFrom(x.vertex) != 0 && !visited[(*i)->other(x.vertex)]){
				q.push(PriV((*i)->other(x.vertex),min(x.priority,(*i)->rCapFrom(x.vertex)),(*i)));
			}
		}
		q.pop();
	}
	delete st[s];
	return false;
}

int Graph::maxFlowBFS(int s, int t){
#ifdef TEST
		augC=0;
		augLenght=0;
		edgesC=0;
#endif

	if (someFlow) clearFlow();
	while (bfs(s,t)) {
#ifdef TEST
		augC++;
#endif
		augment(s,t);
	}
	someFlow=true;
#ifdef TEST
		augC!=0 ? augLenght=augLenght/augC : augLenght=0;
#endif
	return flow(s);
}

int Graph::maxFlowDFS(int s, int t){
#ifdef TEST
		augC=0;
		augLenght=0;
		edgesC=0;
#endif
	if (someFlow) clearFlow();
	while (dfs(s,t)){
#ifdef TEST
		augC++;
#endif
		augment(s,t);
	}
	someFlow=true;
#ifdef TEST
		augC!=0 ? augLenght=augLenght/augC : augLenght=0;
#endif
	return flow(s);
}

int Graph::maxFlowPFS(int s, int t){
#ifdef TEST
		augC=0;
		augLenght=0;
		edgesC=0;
#endif

	if (someFlow) clearFlow();
	while (pfs(s,t)){
#ifdef TEST
		augC++;
#endif
		augment(s,t);
	}
	someFlow=true;	
#ifdef TEST
		augC!=0 ? augLenght=augLenght/augC : augLenght=0;
#endif
	return flow(s);
}

void Graph::computeHeights(int t){
	queue<int> q;
	vector<bool> visited(VCnt(),false);
	q.push(t);
	visited[t]=true;
	height[t]=0;
	while (!q.empty()){
		int x=q.front();
		for(list<Edge*>::iterator i=adj[x].begin();i!=adj[x].end();i++){
			if (!(*i)->from(x) && !visited[(*i)->other(x)] && (*i)->getCap()>0){
				height[(*i)->other(x)]=height[x]+1;
				visited[(*i)->other(x)]=true;
				q.push((*i)->other(x));
			}
		}
		q.pop();
	}
}


int Graph::preflowPush(int s, int t){
	if (someFlow) clearFlow();
	computeHeights(t);
	height[s]=adj.size();
	queue<int> q; 
	vector<int> excess(VCnt(), 0);
	for(list<Edge*>::iterator i=adj[s].begin();i!=adj[s].end();i++){
		if ((*i)->from(s) && (*i)->other(s)!=s){  
			excess[(*i)->other(s)]-=(*i)->rCapFrom(s);
			(*i)->addRFlowFrom(s, (*i)->rCapFrom(s));
			q.push((*i)->other(s));
		}
	}
	while(!q.empty()){
		int x=q.front();
		if (excess[x]==0 || x==s || x==t){  
			q.pop();
			continue;
		}
		int minheight=INT_MAX;	
		for(list<Edge*>::iterator i=adj[x].begin();i!=adj[x].end();i++){
			if (height[x] > height[(*i)->other(x)] && (*i)->rCapFrom(x) > 0){
				int pF = min((*i)->rCapFrom(x),-excess[x]);
				excess[(*i)->other(x)]-=pF;
				excess[x]+=pF;
				(*i)->addRFlowFrom(x, pF);
				if ((*i)->other(x)!=s && (*i)->other(x)!=t && excess[(*i)->other(x)]!=0) {
					q.push((*i)->other(x));
				}
			}
			else{
				if ((*i)->rCapFrom(x) > 0 && height[(*i)->other(x)]<minheight) minheight=height[(*i)->other(x)];
			}
		}
		q.pop();
		if (excess[x]!=0){
//			height[x]++; //pre pripad kedy zvysujeme o 1
			height[x]=minheight+1;
			q.push(x);
		}
	}
	someFlow=true;
	return flow(s);
}


int Graph::highestPreflowPush(int s, int t){
	if (someFlow) clearFlow();
	computeHeights(t);
	height[s]=adj.size(); 
	priority_queue<PriV> q; 
	vector<int> excess(VCnt(), 0);
	for(list<Edge*>::iterator i=adj[s].begin();i!=adj[s].end();i++){
		if ((*i)->from(s) && (*i)->other(s)!=s){  
			excess[(*i)->other(s)]-=(*i)->rCapFrom(s);
			(*i)->addRFlowFrom(s, (*i)->rCapFrom(s));
			q.push( PriV((*i)->other(s),height[(*i)->other(s)],NULL) );
		}
	}
	while(!q.empty()){
		int x=q.top().vertex;
		q.pop();
		if (excess[x]==0 || x==s || x==t){  
			continue;
		}
		int minheight=INT_MAX;	
		for(list<Edge*>::iterator i=adj[x].begin();i!=adj[x].end();i++){
			if (height[x] > height[(*i)->other(x)] && (*i)->rCapFrom(x) > 0){
				int pF = min((*i)->rCapFrom(x),-excess[x]);
				excess[(*i)->other(x)]-=pF;
				excess[x]+=pF;
				(*i)->addRFlowFrom(x, pF);
				if ((*i)->other(x)!=s && (*i)->other(x)!=t && excess[(*i)->other(x)]!=0) {
					q.push( PriV((*i)->other(x),height[(*i)->other(x)],NULL) );
				}
				if (excess[x]==0){
					continue;
				}
			}
			else{
				if ((*i)->rCapFrom(x) > 0 && height[(*i)->other(x)]<minheight) minheight=height[(*i)->other(x)];
			}
		}
		if (excess[x]!=0){
//			height[x]++;
			height[x]=minheight+1;
			q.push( PriV(x,height[x],NULL) );
		}
	}
	someFlow=true;
	return flow(s);
}



int Graph::lowestPreflowPush(int s, int t){
	if (someFlow) clearFlow();
	computeHeights(t);
	height[s]=adj.size(); 
	priority_queue<PriV, vector<PriV>, greater<PriV> > q; 
	vector<int> excess(VCnt(), 0);
	for(list<Edge*>::iterator i=adj[s].begin();i!=adj[s].end();i++){
		if ((*i)->from(s) && (*i)->other(s)!=s){ 
			excess[(*i)->other(s)]-=(*i)->rCapFrom(s);
			(*i)->addRFlowFrom(s, (*i)->rCapFrom(s));
			q.push( PriV((*i)->other(s),height[(*i)->other(s)],NULL) );
		}
	}
	while(!q.empty()){
		int x=q.top().vertex;
		q.pop();
		if (excess[x]==0 || x==s || x==t){ 
			continue;
		}
		int minheight=INT_MAX;
		for(list<Edge*>::iterator i=adj[x].begin();i!=adj[x].end();i++){
			if (height[x] > height[(*i)->other(x)] && (*i)->rCapFrom(x) > 0){
				int pF = min((*i)->rCapFrom(x),-excess[x]);
				excess[(*i)->other(x)]-=pF;
				excess[x]+=pF;
				(*i)->addRFlowFrom(x, pF);
				if ((*i)->other(x)!=s && (*i)->other(x)!=t && excess[(*i)->other(x)]!=0) {
					q.push( PriV((*i)->other(x),height[(*i)->other(x)],NULL) );
				}
				if (excess[x]==0){
					continue;
				}
			}
			else{
				if ((*i)->rCapFrom(x) > 0 && height[(*i)->other(x)]<minheight) minheight=height[(*i)->other(x)];
			}
		}
		if (excess[x]!=0){
//			height[x]++;
			height[x]=minheight+1;
			q.push( PriV(x,height[x],NULL) );
		}
	}
	someFlow=true;
	return flow(s);
}



//GENERATORY


void generuj_grid(Graph &graf, int vx, int vy,int ccap, bool ran=0){
	for (int i=0;i<vx;i++){
		for (int j=0;j<vy;j++){
			if (j<vy-1){
				if (ran) { 
					int cp=rand()%ccap;
					graf.insertO(new Edge(i+j*vx,i+(j+1)*vx,cp));
				}
				else graf.insertO(new Edge(i+j*vx,i+(j+1)*vx,ccap));

				if (ran) {
					int cp=rand()%ccap;
					graf.insertO(new Edge(i+(j+1)*vx,i+j*vx,cp));
				}
				else graf.insertO(new Edge(i+(j+1)*vx,i+j*vx,ccap));

			}
			if (i<vx-1){
				if (ran) { 
					int cp=rand()%ccap;
					graf.insertO(new Edge(i+j*vx,i+1+j*vx,cp));
				}
				else graf.insertO(new Edge(i+j*vx,i+1+j*vx,ccap));

				if (ran) {
					int cp=rand()%ccap;
					graf.insertO(new Edge(i+1+j*vx,i+j*vx,cp));
				}
				else graf.insertO(new Edge(i+1+j*vx,i+j*vx,ccap));
			}
		}
	}
}

//generuj s istou pravd. graf, #vrch, max.kapacita, pravd.prepojenia
void generuj_pravd(Graph &graf, int numV, int ccap, double prPrep, bool ran=0){
	for (int i=0;i<numV;i++){
		for (int j=0;j<numV;j++){
			if (i!=j){ 
				int rp = (int)(prPrep*RAND_MAX);
				if (rand()<rp){
					if (ran) graf.insert(new Edge(i,j,rand()%ccap+1));
					else graf.insert(new Edge(i,j,ccap));
				}
			}
		}
	}

}

//small world graf (g,pocet vrcholov, pocet susednych na prepojenie, kapacita, pravdepodobnost nahodneho prepojenia, rozhodnutie ci pozuit random kapacitu)
void generuj_small_world(Graph &graf, int numV, int neigh ,int ccap, double prPrep=0, bool ran=0){
	int prepojeni=0;
	for (int i=0;i<numV;i++){
		for (int j=-neigh;j<neigh+1;j++){
			if(j!=0){
				int cp=ccap;
				if (ran) cp=rand()%ccap+1;
				graf.insert(new Edge(i,(i+j+numV)%numV,cp));
				if (prPrep>0){
					int rp = (int)(prPrep*RAND_MAX);
					if (rand()<rp){
						int prepoj_kam=rand()%numV;
						if (ran) cp=rand()%ccap+1;
						graf.insert(new Edge(i,prepoj_kam,cp));
						graf.insert(new Edge(prepoj_kam,i,cp)); //toto zmazat pre jednostranne sipy
						prepojeni++;	
					}
				}
			}
		}
	}
	cout<<"prepojeni "<<prepojeni<<endl;
}


void generuj_full(Graph &graf,int numVer,int ccap){
	for (int i=0;i<numVer;i++){
		for (int j=0;j<numVer;j++){
			if (i!=j) graf.insertO(new Edge(i,j,ccap));
		}
	}
}


void generuj_butterfly(Graph &graf,int k,int ccap){
	int kk=(int)pow(2.0,k);
	cout<<"kk "<<kk<<endl;
	for (int i=0;i<k;i++){
		for (int j=0;j<kk;j++){
			graf.insert(new Edge(kk*i+j,kk*(i+1)+j,ccap) );
			graf.insert(new Edge(kk*(i+1)+j,kk*i+j,ccap) ); //neorient
			graf.insert(new Edge(kk*i+j,kk*(i+1)+(j+kk/(int)pow(2.0,i+1))%kk ,ccap) );
			graf.insert(new Edge(kk*(i+1)+(j+kk/(int)pow(2.0,i+1))%kk, kk*i+j ,ccap) ); //neorient
		}
	}
}


void generuj_01(Graph &graf,int numVer){
	for (int i=0;i<numVer;i++){
		for (int j=0;j<numVer;j++){
			graf.insert(new Edge(i,j,(i+j)%2));
		}
	}
}


void generuj_bip(Graph &graf,int part1, int part2, double prPrep){ //zaciatok je v 0, koniec 1, prop==0-1
	for (int i=0;i<part1;i++){
		graf.insertO(new Edge(0,i+2,1));
		for (int j=0;j<part2;j++){
			int rp = (int)(prPrep*RAND_MAX);
			if (rand()<rp)
				graf.insertO(new Edge(i+2,part1+j+2,1));
		}
	}
	for (int j=part1+2;j<part1+part2+2;j++){
		graf.insertO(new Edge(j,1,1));
	}
}



void generuj_reg(Graph &graf,int numVer,int ccap, int deg=3, int ran=0){
	long long done=0;
	vector<vector<int> > a(numVer);
	long long pzba=(long long)deg*numVer;
	pzba/=2;
	long long cyklov=0;
	while(done<pzba){
		cyklov++;
		int v1=(rand()*rand())%numVer;
		while (a[v1].size()>=deg){
			v1++;
			v1%=numVer;
		}
		int v2=(rand()*rand())%numVer;
		int v2z=v2;
		bool ok=false;
		do{
			v2++;
			v2%=numVer;
			if (v1==v2 || a[v2].size()>=(deg)) continue;
			for (int i=0;i<a[v1].size();i++){
				if (a[v1][i]==v2) continue;
			}
			a[v1].push_back(v2);
			a[v2].push_back(v1);
			done++;
			ok=true;
		}while (!ok && v2!=v2z);
		if (!ok){
			for (int i=0;i<5;i++){
				int vv=(rand()*rand())%numVer;
				if (a[vv].size()){
					int vb=a[vv][0];				
					for (int j=0;j<a[vb].size();j++){
						if (a[vb][j]==vv){
							a[vb][j]=a[vb][a[vb].size()-1];
							a[vv][0]=a[vv][a[vv].size()-1];
							a[vb].resize(a[vb].size()-1);
							a[vv].resize(a[vv].size()-1);
							done--;
							break;
						}
					}
				}
			}
		}
		
	}
	for (int i=0;i<numVer;i++){
		for (int j=0;j<3;j++){
			if (ran) graf.insert(new Edge(a[i][j],i,rand()%(ccap)+1));
			else graf.insert(new Edge(a[i][j],i,ccap));			
		}
	}
}

int countEdges(Graph& graf){
	int out=0;
	for(int i=0;i<graf.adj.size();i++){
		iter(graf.adj[i],ii){
			if ((*ii)->from(i)) out++;
		}
	}
	return out;
}



void testovanie_full(){
	ofstream fout ("vystup.txt");
	int ss=0, tt=1;
	clock_t start, end;
	int poctestov=11;
	for (int i=20;i>2;i--) {
		vector< vector <double> > vysl(5,vector<double>(poctestov,0)); 
		int nv=5*i;
		Graph graf(nv);
//		generuj_full(graf,nv,100);
		generuj_pravd(graf, nv, 50, 1,1);

		for (int j=0;j<poctestov;j++) {
			ss=rand()%nv;
			tt=rand()%nv;
			if (ss==tt) ss++;
			set<int> toky;
			if (j==0) fout<<nv<<" ";
			start=clock();	toky.insert(graf.maxFlowBFS(ss,tt));	end=clock();		vysl[0][j]=(((end-start) / (double)CLOCKS_PER_SEC));
			start=clock();	toky.insert(graf.maxFlowDFS(ss,tt));	end=clock();		vysl[1][j]=(((end-start) / (double)CLOCKS_PER_SEC));
			start=clock();	toky.insert(graf.maxFlowPFS(ss,tt));	end=clock();		vysl[2][j]=(((end-start) / (double)CLOCKS_PER_SEC));
			start=clock();	toky.insert(graf.preflowPush(ss,tt));	end=clock();		vysl[3][j]=(((end-start) / (double)CLOCKS_PER_SEC));
			start=clock();  toky.insert(graf.highestPreflowPush(ss,tt));	end=clock();	vysl[4][j]=(((end-start) / (double)CLOCKS_PER_SEC));
			if (toky.size()!=1) cout<<"!!!!!"<<endl; else cout<<*toky.begin()<<" ";
		}
		for (int j=0;j<5;j++) {
			sort(vysl[j].begin(),vysl[j].end());
			fout << vysl[j][poctestov/2];
			if (j==4) fout << endl;
			else fout << " ";
		}
	
	}
}

void vykrelsenie_pravd(){
	int ss=0, tt=1;
	clock_t start, end;
	int nv=23;
	Graph graf(nv);
	cout<<nv<<" ";
	set<int> toky;
	generuj_pravd(graf, nv, 100, 0.15,1);
	start=clock();	toky.insert(graf.maxFlowBFS(ss,tt));	end=clock();		cout << ((end-start) / (double)CLOCKS_PER_SEC)<<" ";
	graf.graphviz_di("graph1.dot",ss,tt);
	start=clock();	toky.insert(graf.maxFlowDFS(ss,tt));	end=clock();		cout << ((end-start) / (double)CLOCKS_PER_SEC)<<" ";
	graf.graphviz_di("graph2.dot",ss,tt);	
	start=clock();	toky.insert(graf.maxFlowPFS(ss,tt));	end=clock();		cout << ((end-start) / (double)CLOCKS_PER_SEC)<<" ";
	graf.graphviz_di("graph3.dot",ss,tt);	
	start=clock();	toky.insert(graf.preflowPush(ss,tt));	end=clock();		cout << ((end-start) / (double)CLOCKS_PER_SEC)<<" ";
	graf.graphviz_di("graph4.dot",ss,tt);	
	start=clock();  toky.insert(graf.highestPreflowPush(ss,tt));	end=clock();	cout << ((end-start) / (double)CLOCKS_PER_SEC)<<endl;
	graf.graphviz_di("graph5.dot",ss,tt);
	if (toky.size()!=1) cout<<"!!!!!"<<endl; else cout<<*toky.begin()<<" ";
}

#ifdef TEST
void test_all(){
	int ss=0, tt=1;
	clock_t start, end;
	int nv=60;
	Graph graf(nv);
	cout<<nv<<" ";
	set<int> toky;
	generuj_pravd(graf, nv, 600, 0.1);
	start=clock();	toky.insert(graf.maxFlowBFS(ss,tt));	end=clock();		cout << ((end-start) / (double)CLOCKS_PER_SEC)<<" ";
	graf.graphviz_di("graph1.dot",ss,tt);

	cout<< endl << "bfs | " <<  graf.augC << " | " << graf.augLenght << " | " << graf.edgesC << endl;


	start=clock();	toky.insert(graf.maxFlowDFS(ss,tt));	end=clock();		cout << ((end-start) / (double)CLOCKS_PER_SEC)<<" ";
	graf.graphviz_di("graph2.dot",ss,tt);	

	cout<< endl << "dfs | " <<  graf.augC << " | " << graf.augLenght << " | " << graf.edgesC << endl;

	start=clock();	toky.insert(graf.maxFlowPFS(ss,tt));	end=clock();		cout << ((end-start) / (double)CLOCKS_PER_SEC)<<" ";
	graf.graphviz_di("graph3.dot",ss,tt);	

	cout<< endl << "pfs | " <<  graf.augC << " | " << graf.augLenght << " | " << graf.edgesC << endl;

	start=clock();	toky.insert(graf.preflowPush(ss,tt));	end=clock();		cout << ((end-start) / (double)CLOCKS_PER_SEC)<<" ";
	graf.graphviz_di("graph4.dot",ss,tt);	

	cout<< endl << "FIFO preflow | " <<  graf.pushC << " | " << graf.relabelC << " | " << graf.edgesC << endl;
	

	start=clock();  toky.insert(graf.highestPreflowPush(ss,tt));	end=clock();	cout << ((end-start) / (double)CLOCKS_PER_SEC)<<endl;
	graf.graphviz_di("graph5.dot",ss,tt);

	cout<< endl << "Highest Vertex preflow | " <<  graf.pushC << " | " << graf.relabelC << " | " << graf.edgesC << endl;

	
	if (toky.size()!=1) cout<<"!!!!!"<<endl; else cout<<*toky.begin()<<" ";
	cout<<"!!! "<<graf.augC;
}
#endif

int main(){
#ifdef TEST
	cout<<"!!!!!!!!POZOR TESTUJU SA POCTY OPERACII!!!!!!"<<endl;
#endif
	srand( time(NULL) );
}


























